'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.init = init;
exports.mount = mount;
exports.unmount = unmount;
var types = {};

var __cache = {};

function log(level, msg) {
  var _console;

  for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  (_console = console)[level].apply(_console, ['⚙️ micromanager -', msg].concat(args));
}

function init(t) {
  types = t;
}

function mount(_ref) {
  var _ref$micromanagerRoot = _ref.micromanagerRoot,
      micromanagerRoot = _ref$micromanagerRoot === undefined ? 'micromanagerRoot' : _ref$micromanagerRoot;

  for (var type in types) {
    var attr = 'data-' + type;
    var nodes = [].slice.call(document.querySelectorAll('[' + attr + ']'));
    var path = types[type].replace(/^\/|\/$/, '');

    for (var i = 0; i < nodes.length; i++) {
      var name = nodes[i].getAttribute(attr);

      try {
        var instance = require(micromanagerRoot + '/' + path + '/' + name + '.js').default(nodes[i]);

        nodes[i].removeAttribute(attr);

        if (instance) {
          this.cache.set(instance.displayName || name, instance);
        }
      } catch (e) {
        log('error', name + ' threw an error\n\n', e);
      }
    }
  }
}

function unmount() {
  for (var key in __cache) {
    var instance = __cache[key];
    if (instance.unmount) {
      instance.unmount();
      delete __cache[key];
    }
  }
}

var cache = exports.cache = {
  set: function set(id, instance) {
    if (__cache[id]) log('warn', 'a duplicate key ' + id + ' was found in the cache. This instance will be overwritten.');
    __cache[id] = instance;
  },
  get: function get(id) {
    try {
      return __cache[id];
    } catch (e) {
      log('warn', 'can\'t find ' + id + ' in the cache', e);
      return null;
    }
  },
  dump: function dump() {
    return __cache;
  }
};